# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ..core.datetime_utils import serialize_datetime

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class ChatConnector(pydantic.BaseModel):
    """
    The connector used for fetching documents.
    """

    id: str = pydantic.Field(description="The identifier of the connector. Currently only 'web-search' is supported.")
    user_access_token: typing.Optional[str] = pydantic.Field(
        description="An optional override to set the token that Cohere passes to the connector in the Authorization header."
    )
    continue_on_failure: typing.Optional[bool] = pydantic.Field(
        description="An optional override to set whether or not the request continues if this connector fails."
    )
    options: typing.Optional[typing.Dict[str, typing.Any]] = pydantic.Field(
        description=(
            "Provides the connector with different settings at request time. The key/value pairs of this object are specific to each connector.\n"
            "\n"
            "The supported options are:\n"
            "\n"
            "**web-search**\n"
            "\n"
            "**site** - The web search results will be restricted to this domain (and TLD) when specified. Only a single domain is specified, and subdomains are also accepted.\n"
            "Examples:\n"
            "\n"
            '- `{"options": {"site": "cohere.com"}}` would restrict the results to all subdomains at cohere.com\n'
            '- `{"options": {"site": "txt.cohere.com"}}` would restrict the results to `txt.cohere.com`\n'
        )
    )

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        json_encoders = {dt.datetime: serialize_datetime}
